defmodule Amethyst.GameCoordinator do
  use GenServer

  @moduledoc """
  The game coordinator is responsible for keeping track of active
  instances of each game and can create new ones on demand.
  """

  @impl true
  def init(initial) do
    {:ok, initial}
  end

  @impl true
  def handle_call({:get, type}, _from, state) do
    {:reply, find_or_create(type, state), state}
  end

  defp create(mod) do
    state = mod.initialize()
    spawn(mod, :listen, [state])
  end

  defp find(type, games) do
    existing = games |> Enum.filter(fn {mod, _pid, _opts} -> mod == type end)
    [{_mod, pid, _} | _] = existing
    pid
  end

  defp find_or_create(type, games) do
    case find(type, games) do
      nil -> create(type)
      some -> some
    end
  end
end
