defmodule Amethyst.GameRegistry do
  use GenServer

  @moduledoc """
  The game registry stores information about which games are
  available.
  """

  @impl true
  def init(initial) do
    {:ok, initial}
  end

  @impl true
  def handle_call({:register, module, meta}, _, state) do
    if Keyword.get(meta, :default, false) && find_default(state) != nil do
      {:reply, :error, :default_exists}
    end
    {:reply, :ok, [{module, meta} | state]}
  end

  @impl true
  def handle_call({:list}, _, state) do
    {:reply, state, state}
  end

  @impl true
  def handle_call({:get_default}, _, state) do
    {:reply, find_default(state), state}
  end

  defp find_default(state) do
    state |> Enum.filter(fn {_mod, meta} ->
      Keyword.get(meta, :default, false)
    end) |> List.first(nil)
  end

  def start_link(initial) when is_list(initial) do
    GenServer.start_link(__MODULE__, initial, name: {:global, __MODULE__})
  end

  def register(module, meta) do
    GenServer.call({:global, __MODULE__}, {:register, module, meta})
  end

  def list() do
    GenServer.call({:global, __MODULE__}, {:list})
  end

  def get_default() do
    GenServer.call({:global, __MODULE__}, {:get_default})
  end
end
